# frozen_string_literal: true

module Lipstick
  module Helpers
    module LayoutHelper
      def aaf_header(title:, environment: nil, auth: nil, &blk)
        content_tag('div', class: 'aaf-header') do
          concat(aaf_banner(title, environment, auth))
          concat(capture(&blk))
        end
      end

      def aaf_footer(&block)
        content_tag('footer') do
          content_tag('div', class: 'footer-content') do
            concat(aaf_logo)
            concat(capture(&block))
          end
        end
      end

      def logged_in_user(user)
        return if user.nil?

        content_tag('p') do
          concat('Logged in as: ')
          concat(content_tag('strong', user.name))
          concat(" (#{user.try(:targeted_id)})") if user.try(:targeted_id)
        end
      end

      def page_header(header, subheader = nil)
        content_tag('div', class: 'page-header') do
          content_tag('h1') do
            concat(header)
            concat(' ')
            concat(content_tag('small', subheader)) if subheader
          end
        end
      end

      def yes_no_string(boolean)
        boolean ? 'Yes' : 'No'
      end

      def icon_tag(icon_class, html_opts = {})
        html_opts[:class] =
          "#{html_opts[:class]} glyphicon glyphicon-#{icon_class}".strip
        content_tag('span', '', html_opts)
      end

      # button_link_to(url_opts) { 'Link Text' }
      # button_link_to(url_opts, html_opts) { 'Link Text' }
      # button_link_to('Link Text', url_opts)
      # button_link_to('Link Text', url_opts, html_opts)
      def button_link_to(*args, &block)
        args.unshift(capture(&block)) if block_given?
        text, url_opts, html_opts = args
        html_opts ||= {}
        html_opts[:class] ||= 'btn-default'
        html_opts[:class] = "#{html_opts[:class]} btn".strip
        link_to(text, url_opts, html_opts)
      end

      def info_message(title, &block)
        alert_block(title, 'info', &block)
      end

      def error_message(title, &block)
        alert_block(title, 'danger', &block)
      end

      def success_message(title, &block)
        alert_block(title, 'success', &block)
      end

      def warning_message(title, &block)
        alert_block(title, 'warning', &block)
      end

      def breadcrumbs(*links)
        content_tag('ol', class: 'breadcrumb') do
          last = links.pop

          links.each do |link|
            concat(content_tag('li', breadcrumb_link(link)))
          end

          concat(content_tag('li', breadcrumb_link(last),
                             class: 'active'))
        end
      end

      def will_paginate(_type = nil, options = {})
        options[:renderer] ||= Lipstick::Helpers::PaginationLinkRenderer
        super
      end

      DISABLE_ANIMATIONS_CSS = <<-CSS
        .ui, .ui * {
          -webkit-animation-duration: 0ms !important;
          -moz-animation-duration: 0ms !important;
          -ms-animation-duration: 0ms !important;
          animation-duration: 0ms !important;
        }
      CSS
      private_constant :DISABLE_ANIMATIONS_CSS

      def disable_animations
        content_tag('style', DISABLE_ANIMATIONS_CSS, type: 'text/css')
      end

      private

      def aaf_banner(title, environment, auth)
        capture do
          text = content_tag('header') do
            concat(aaf_links(auth))
            concat(title)
            concat(aaf_environment_string(environment))
          end
          concat(text)
        end
      end

      def aaf_logo
        content_tag('span', '', class: 'pull-right logo hidden-xs hidden-sm')
      end

      def aaf_links(auth)
        content_tag('div', class: 'aaf-links pull-right') do
          concat(aaf_home_link)
          concat(aaf_support_link)
          concat(aaf_auth_link(auth))
        end
      end

      def aaf_home_link
        content_tag('a', href: 'https://aaf.edu.au') do
          concat(icon_tag('aaf'))
          concat(' AAF Home')
        end
      end

      def aaf_support_link
        content_tag('a', href: 'https://support.aaf.edu.au') do
          concat(icon_tag('user'))
          concat(' Support')
        end
      end

      def aaf_auth_link(auth)
        return if auth.nil?

        text = ' Log In'
        text = ' Log Out' if auth == :logout

        icon = 'log-in'
        icon = 'log-out' if auth == :logout

        content_tag('a', href: "/auth/#{auth}") do
          concat(icon_tag(icon))
          concat(text)
        end
      end

      def aaf_environment_string(environment)
        return unless environment&.present?

        content_tag('span', environment, class: 'environment')
      end

      def alert_block(title, color_class, &block)
        opts = { class: "alert alert-#{color_class}", role: 'alert' }
        content_tag('div', opts) do
          concat(content_tag('h4', title))
          concat(capture(&block))
        end
      end

      def breadcrumb_link(item)
        return item if item.is_a?(String)

        item.reduce(nil) { |_, (k, v)| content_tag('a', k, href: v) }
      end
    end
  end
end
