# frozen_string_literal: true

module Lipstick
  module Images
    # Creates a logo-only email banner, with two elements.
    #
    # 1. Logo, centered vertically and horizontally within the image.
    # 2. Environment string, stamped across the logo.
    #
    # The math here seemed like it would be a lot simpler than the EmailBanner
    # class, but then I went and made it all confusing by bringing trigonometry
    # into it. I promise there was a good reason for doing it.
    class LogoOnlyEmailBanner
      def initialize(image:, environment:, background_color:)
        require 'rmagick'

        @environment = environment.upcase
        @image = image
        @background_color = background_color
      end

      def to_png
        bgcolor = @background_color

        canvas = Magick::ImageList.new
        canvas.new_image(WIDTH, HEIGHT) do |f|
          f.format = 'PNG'
          f.background_color = bgcolor
        end

        canvas.composite!(logo, logo_x, logo_y, Magick::SrcOverCompositeOp)

        annotate_environment(canvas)

        canvas.to_blob
      end

      private

      WIDTH = 600
      HEIGHT = 150
      MID_Y = HEIGHT / 2
      ANGLE_DEGREES = 10
      ANGLE_RADIANS = Math::PI * ANGLE_DEGREES / 180
      private_constant :WIDTH, :HEIGHT, :MID_Y, :ANGLE_DEGREES, :ANGLE_RADIANS

      def environment_format
        @environment_format ||= Magick::Draw.new do |title|
          title.font_family = 'ArialB'
          title.font_weight = 900
          title.pointsize = 36
          title.gravity = Magick::ForgetGravity
          title.fill = '#bc2028'
          title.rotation = -1 * ANGLE_DEGREES
        end
      end

      def blank_environment?
        !@environment&.present?
      end

      def annotate_environment(canvas)
        return if blank_environment?

        environment_format.annotate(canvas, 0, 0, env_x, env_y, @environment)
      end

      def logo
        @logo ||= Magick::Image.read(@image.pathname.to_s).first
      end

      def gap
        @gap ||= environment_format.get_type_metrics('AAF').width
      end

      def logo_w
        logo.bounding_box.width
      end

      # x position to vertically center logo
      def logo_x
        ((WIDTH - logo.bounding_box.width) / 2).round
      end

      # y position to vertically center logo
      def logo_y
        ((HEIGHT - logo.bounding_box.height) / 2).round
      end

      def env_metrics
        return OpenStruct.new(width: 0, height: 0) if blank_environment?

        environment_format.get_type_metrics(@environment)
      end

      def env_x
        m = env_metrics

        # The horizontal width of the environment text is found by:
        # w cos(0) + h sin(0)
        #
        # w = text width
        # h = text ascent
        # 0 = angle of text rotation
        #
        # Contrary to the description in `EmailBanner`, since we're upper
        # casing the environment string here, we don't need to worry about text
        # descent.
        w = Math.cos(ANGLE_RADIANS) * m.width +
            Math.sin(ANGLE_RADIANS) * m.ascent

        # We center that the same as for `logo_x`
        ((WIDTH - w) / 2).round
      end

      def env_y
        m = env_metrics

        # The horizontal width of the environment text is found by:
        # w sin(0) + h cos(0)
        h = Math.sin(ANGLE_RADIANS) * m.width +
            Math.cos(ANGLE_RADIANS) * m.ascent

        # We use addition here instead of the usual subtraction, as the text
        # moves in the upward direction from the position we set.
        ((HEIGHT + h) / 2).round
      end
    end
  end
end
